#ifndef __DOCRA_PROJECT_HPP
#define __DOCRA_PROJECT_HPP

#include <string>

#include "parser.hpp"
#include "util.hpp"
#include "fs.hpp"

namespace docra {//docra``+

  struct Project //+`Project` Aggregates comments collected from all files found in the command line input
  {
    const Config &_cfg;

    std::string base_url;

    Project(const Config &cfg);

    void set_base_url(const std::string &bu) { base_url = bu; }

    //++`::analyze_directory`
    // Recursively scan a directory looking for cigma tags
    //>`path` Directory to scan
    //>`level` Relative depth of path in source hierarchy
    //-> 'false' on error, 'true' otherwise
    bool analyze_directory(int verb, const fs::path &path, int level); //?>

    //++`::analyze_file`
    // Lex and parse the given file
    //>`path` Path to file
    //>`level` Relative depth of path in source hierarchy
    //-> 'false' on error, 'true' otherwise
    bool analyze_file(int verb, const fs::path &path, int level); //?>

    Prefixes prefixes;

    //++`::scopes` Association of each file path in the project with its syntax tree
    FileScopes scopes;//?>

    //++`.insert_tag`
    Suffix *insert_tag(const Tag &tag, Token &&token, Scope *scope);//?>

    //++`.locate_prefix`
    Prefix *locate_prefix(const std::string &prefix);//?>

    //++`.locate_symbol`1
    Symbol *locate_symbol(const std::string &prefix, const std::string &symbol);//?>

    //++`.locate_symbol`2
    Symbol *locate_symbol(const Tag &tag);//?>

    //++`.locate`1
    Suffix *locate(const std::string &prefix, const std::string &symbol, const std::string &suffix = "");//?>

    //++`.locate`2
    Suffix *locate(const Tag &tag);//?>

    void debug();
  };
}

#endif//__DOCRA_PROJECT_HPP
