#ifndef __TESTS_TEST_HPP
#define __TESTS_TEST_HPP

#include <gtest/gtest.h>
#include <docra.hpp>
#include <map>
#include <memory>

class TestNoFile : public ::testing::Test {
protected:
  docra::Lexer_Dummy lexer;
};

template <class L>
class TestTmpFile : public ::testing::Test {
protected:
  FILE *tmp;
  L lexer;
  docra::Config cfg;
  docra::Project project;
  docra::Parser_Source parser;

  TestTmpFile() : tmp(tmpfile()), project(cfg), parser(lexer, project) {
    lexer.init("<test>", tmp);
  }

  ~TestTmpFile() {
    if (tmp) fclose(tmp);
  }

public:
  void ws(const char *str) {
    fputs(str, tmp);
  }
  void wl(const char *str) {
    ws(str); fputs("\n", tmp);
  }
  void we() {
    fseek(tmp, 0, SEEK_SET);
  }

  void rewind() {
    ::rewind(tmp);
  }
};

// Assert Chunk

#define CASSERT(s, t, d) \
  ASSERT_TRUE(lexer.next_chunk()); \
  ASSERT_EQ(lexer.chunk.kind, t);  \
  ASSERT_EQ(lexer.chunk.str, s);   \
  ASSERT_EQ(lexer.chunk.depth, d)

#define CASSERT_CODE(s, d) CASSERT(s, docra::Chunk::CODE, d)
#define CASSERT_COMM(s, d) CASSERT(s, docra::Chunk::COMMENT, d)

#define CASSERT_EOF() \
  ASSERT_FALSE(lexer.next_chunk())

// Assert Token

#define TASSERT_CODE(c) \
  ASSERT_TRUE(lexer.next_token()); \
  ASSERT_EQ(lexer.token.kind, docra::Token::CODE); \
  if (c != nullptr) { std::string cs(c); ASSERT_EQ(lexer.token.str, cs); }

#define TASSERT_STR(s) \
  ASSERT_TRUE(lexer.next_token()); \
  ASSERT_EQ(lexer.token.kind, docra::Token::DOC_STR); \
  if (s != nullptr) { std::string ss(s); ASSERT_EQ(lexer.token.str, ss); }

#define TASSERT_SIN() \
  ASSERT_TRUE(lexer.next_token()); \
  ASSERT_EQ(lexer.token.kind, docra::Token::SCOPE_ENTER);

#define TASSERT_SOUT() \
  ASSERT_TRUE(lexer.next_token()); \
  ASSERT_EQ(lexer.token.kind, docra::Token::SCOPE_EXIT);

#define TASSERT_BIN() \
  ASSERT_TRUE(lexer.next_token()); \
  ASSERT_EQ(lexer.token.kind, docra::Token::BLOCK_ENTER);

#define TASSERT_BOUT() \
  ASSERT_TRUE(lexer.next_token()); \
  ASSERT_EQ(lexer.token.kind, docra::Token::BLOCK_EXIT);

#define TASSERT_TAG(p, y, s) \
  ASSERT_TRUE(lexer.next_token()); \
  ASSERT_EQ(lexer.token.kind, docra::Token::TAG); \
  if (p != nullptr) { std::string ps(p); ASSERT_EQ(lexer.token.tag.prefix, ps); } \
  if (y != nullptr) { std::string ys(y); ASSERT_EQ(lexer.token.tag.symbol, ys); } \
  if (s != nullptr) { std::string ss(s); ASSERT_EQ(lexer.token.tag.suffix, ss); }

#define TASSERT_EOL() \
  ASSERT_TRUE(lexer.next_token()); \
  ASSERT_EQ(lexer.token.kind, docra::Token::EOL);

#define TASSERT_EOF() \
  ASSERT_FALSE(lexer.next_token());

struct passert_node;
typedef std::map<std::string, std::unique_ptr<passert_node>> passert_map;

#endif//__TESTS_TEST_HPP
